;************************************************************************************
;Author:        Limerick MicroConverter Applications Group
;
;Description:   To interface an ADM1032 temperature sensor to the ADuC814 via
;               I2C. Data is read back from the ADuC814 through the RS232 interface.
;************************************************************************************	
$MOD814

BITCNT          DATA    30h   ;bit counter for I2C routines
SLAVEADD        DATA    31h   ;slave address for I2C routines
INPUT           DATA    32h   ;data recieved from the slave
OUTPUT          DATA    33h   ;data to be transmitted to slave
NOACK           BIT     00h   ;I2C no acknowledge flag
ERR             BIT     00h   ;I2C error flag
LED             EQU     P3.4


CSEG
ORG 0000h
        JMP MAIN

ORG 0060h
MAIN:
                              
        MOV RCAP2H,#0ffh  	;Configure Baud Rate for 9600.
        MOV RCAP2L,#-7    	;
        MOV TH2,-1
        MOV TL2,-7
        MOV SCON,#52h
        MOV T2CON,#34h
              
        MOV CFG814,#01H       ;enable i2c pins on 814
     
        MOV SLAVEADD,#98H     ;Address of ADM1032 is default at 4Ch. Therefore shift 4Ch
                              ;left by 1 and add the R/W bit to the LSB....... 
                              ;=>01001100b becomes 1001100R/W for transmission by Master.
                              ;Right-shift by 1 for slave interpretation......
                              ;if Slave receives 99h,it removes LSB (R/W bit), shifts top
                              ;7 bits to the right by 1 and adds a 0 to the MSB = 4Ch.
                              ;The original LSB is taken off as the Read/Write control.
                              ;If LSB=0 this means WRITE. If LSB=1 this means READ. 
        MOV I2CCON,#0A8h      ;Sets SDATA & SCLOCK, and selects master mode.
                              ;MDO (I2CCON.7) sets Master TX I2C in software mode.
                              ;data written to MDO will be outputted on SDATA if MDE
                              ;(I2CCON.6) =1. If MDE=0 then SDATA/MDO acts as an input.
                              ;MCO (I2CCON.5) is the SCLOCK OUT bit. Data written to this
                              ;will appear on SCLOCK.
                              ;I2CM (I2CCON.3) sets software master mode is = 1.
                              ;Clearing this sets Hardware slave mode.
 
        MOV OUTPUT,#00h       ;TX 0 via I2C, Address of Local Temp sensor register on ADM1032

        CLR NOACK
        CLR ERR

;code for write mode ( master transmits one byte to slave )
SEND:
        CALL SENDDATA 	      ;sends start bit
                              ;sends address byte
                              ;checks acknowledge
                              ;transmits ACC
                              ;checks ACK
                              ;sends stop bit

RXTXLOOP:    
;code for a read mode ( master recieves one byte from slave )
        CALL RCVDATA          ;sends start bit
                              ;sends address byte
                              ;checks acknowledge
                              ;receives byte into ACC
                              ;checks ACK
                              ;sends stop bit
;Check for Error message
        JB ERR,SENDERR        ;if error, send error message

;Transmit received byte (INPUT) up UART to PC (hyperterminal)
        MOV A,INPUT           ;put value received into ACC
        CALL SENDVAL          ;send value received out the UART
        JMP SKIP

SENDERR:
        CALL ERROR            ;send error message out the UART
        CLR  ERR              ;clear error flag

SKIP:	  MOV A,#10             ;send LF+CR
        CALL SENDCHAR
        MOV  A,#13
        CALL SENDCHAR

;Toggle LED (~100ms delay so that LED can be seen toggle & data read)
tgl:    MOV A, #1 		;Sets up 100ms * A
 	  CALL DELAY            ;Delay is to allow LED be seen and to regulate 
        CPL  LED              ;rate of data return via RS232

        jmp send              ;Repeat measurement.

;====================================================================
;                             SUBROUTINES
;====================================================================
;Send all the sequence to the slave (slave address + data (OUTPUT))
SENDDATA:
        				;send start bit
        CALL STARTBIT         ;acquire bus and send slave address
        				;send slave address
        MOV  A, SLAVEADD
        CALL SENDBYTE        	;sets NOACK if NACK received
        JB   NOACK, STOPSEND 	;if no acknowledge send stop
        				;send OUTPUT byte
        MOV  A, OUTPUT
        CALL SENDBYTE         ;sets NOACK if NACK received
STOPSEND:  
        CALL STOPBIT          ;sends stop bit
        JNB  NOACK, SENDRET   ;if slave sends NACK send error
        SETB ERR              ;sets the error flag
SENDRET:
        RET
;**************************************************************************************
; receives one or more bytes of data from an I2C slave device.
RCVDATA:   
 	  INC SLAVEADD        	;Set RW for reception....i.e. set LSB of shifted address
        	  			;send start bit
        CALL STARTBIT         ;acquire bus and send slave address
        				;send slave address
        MOV  A, SLAVEADD	;READ BIT ALREADY SET. ADDRESS=4Ch = ADM1032 product.
        CALL SENDBYTE         ;sets NOACK if NACK received
        DEC  SLAVEADD         ;returns SLAVEADD to 98h (after INC), shifted address 4Ch
					;which is the address of the ADM1032
        JB   NOACK, STOPRCV   ;Check for slave not responding.
        CALL RCVBYTE          ;Receive next data byte.
        MOV  INPUT,A       	;Save data byte in buffer.
STOPRCV:
        CALL STOPBIT
        JNB  NOACK, RCVRET    ;if slave sends NACK send error
        SETB ERR              ;sets the error flag
RCVRET:
        RET
;***************************************************************************************
; Sends the start bit to initiate an I2C communication
;Start is defined as a low going edge on SDATA during an SCLOCK high time.
STARTBIT:
        SETB MDE             ;enable SDATA pin as an output
        CLR  NOACK
        CLR  MDO             ;low O/P on SDATA 
        CLR  MCO             ;start bit
        RET
;***************************************************************************************
; Sends the stop bit to end an I2C transmission
;STOP is defined as a HIGH going edge on SDATA during an SCLOCK high time.
STOPBIT:
        SETB MDE             ;to enable SDATA pin as an output
        CLR  MDO             ;get SDATA ready for stop
        SETB MCO             ;set clock for stop
        SETB MDO             ;this is the stop bit
        RET
;***************************************************************************************
; Send 8-bits in ACC to the slave
SENDBYTE:
        MOV  BITCNT,#8       ;8 bits in a byte
        SETB MDE             ;to enable SDATA pin as an output
        CLR  MCO             ;make sure that the clock line is low
SENDBIT:
        RLC A                ;put data bit to be sent into carry
        MOV MDO,C            ;put data bit on SDATA line
        SETB MCO             ;clock to send bit
        CLR  MCO             ;clear clock 
        DJNZ BITCNT,SENDBIT  ;jump back and send all eight bits

        CLR  MDE             ;release data line for acknowledge
        SETB MCO             ;send clock for acknowledge
        JNB  MDI,NEXT        ;this is a check for acknowledge
        SETB NOACK           ;no acknowledge, set flag
NEXT:   CLR  MCO             ;clear clock 
        RET
;***************************************************************************************
; receives one byte of data from an I2C slave device. Returns it in A
RCVBYTE:
        MOV BITCNT,#8        ;Set bit count.
        CLR MDE              ;to enable SDATA pin as an input	
        CLR MCO              ;make sure the clock line is low
RCVBIT:  
        SETB MCO             ;clock to recieve bit
        CLR  MCO             ;clear clock
        MOV  C,MDI           ;read data bit into carry.
        RLC  A               ;Rotate bit into result byte.
	  DJNZ BITCNT,RCVBIT   ;Repeat until all bits received.
                             ;recieved byte is in the accumulator
        SETB MDE             ;Data pin =Output for NACK
        SETB MDO             ;Send NACK (always send NACK for
                             ;last byte in transmission)
        SETB    MCO          ;Send NACK clock.
        CLR     MCO
        RET
;***************************************************************************************
; DELAY ROUTINE FOR THE ADuC814
DELAY:                       ;Delays by 100ms * A
				     ;ADuC812 100ms based on 11.0592MHz Core Clock
				     ;ADuC824 100ms based on 1.573MHz Core Clock 
        MOV R2,A             ;Acc holds delay variable
DLY0:   MOV R3,#65           ;Set up delay loop0
DLY1:   MOV R4,#200          ;Set up delay loop1
        DJNZ R4,$            ;Dec R4 & Jump here until R4 is 0
                             ;wait here for 131*15.3us=2ms
        DJNZ R3,DLY1         ;Dec R3 & Jump DLY1 until R3 is 0
                             ;Wait for 50*2ms
        DJNZ R2,DLY0         ;Dec R2 & Jump DLY0 until R2 is 0
                             ;wait for ACC*100ms
        RET                  ;Return from subroutine
;***************************************************************************************
; this subroutine is run if a NACK is received from the slave
ERROR:
        MOV  A,#45h	
	  CALL SENDCHAR	     ;send the letter E out the UART
	  RET
;***************************************************************************************				
SENDCHAR:       		     ;sends ASCII value contained in A to UART
        JNB TI,$             ;wait til present char gone
        CLR TI               ;must clear TI
        MOV SBUF,A
        RET
;***************************************************************************************
; converts A into the hex character representing the value of A's 
; least significant nibble
HEX2ASCII:      
        ANL  A,#00Fh
        CJNE A,#00Ah,$+3
        JC   IO0030
        ADD  A,#007h
IO0030: ADD  A,#'0'
        RET
;***************************************************************************************
; converts the hex value of A into two ASCII chars, and then spits 
; these two characters up the UART. does not change the value of A.
SENDVAL: 
        PUSH ACC
        SWAP A
        CALL HEX2ASCII
        CALL SENDCHAR        ;send high nibble
        POP  ACC
        PUSH ACC
        CALL HEX2ASCII
        CALL SENDCHAR        ;send low nibble
        POP  ACC
        RET
;____________________________________________________________________

END
